@@ -85,6 +85,14 @@ EdgeColor = wx.Colour( 64, 64, 64 )
 class ToolkitEditorFactory(BaseToolkitEditorFactory):
     """ wxPython editor factory for color editors.
     """
+    
+    # Should the text editor show the text of the color in the text box? 
+    # The background is already showing the color so the text may be
+    # distracting or cut off.  To use this trait, a custom editor must
+    # be created with this trait set, and the style='text'.  
+    # TODO: Think about easier ways to expose this, or changing the default.
+    show_text = Bool ( True )
+    
     #---------------------------------------------------------------------------
     #  Gets the wxPython color equivalent of the object trait:
     #---------------------------------------------------------------------------
@@ -93,9 +101,15 @@ class ToolkitEditorFactory(BaseToolkitEditorFactory):
         """ Gets the wxPython color equivalent of the object trait.
         """
         if self.mapped:
-            return getattr( editor.object, editor.name + '_' )
-    
-        return getattr( editor.object, editor.name )
+            attr = getattr( editor.object, editor.name + '_' )
+        else:
+            attr = getattr( editor.object, editor.name )
+            
+        if isinstance(attr, tuple):
+            attr = wx.Colour( *[ int( round( c * 255.0 ) )
+                             for c in attr ] )
+        return attr
+        
      
     #---------------------------------------------------------------------------
     #  Gets the application equivalent of a wxPython value:
@@ -153,7 +167,10 @@ class SimpleColorEditor ( Editor ):
             editor.
         """
         if not self._no_update:
-            self.selector.color = self.factory.to_wx_color( self )
+            color = self.factory.to_wx_color( self )
+            self.selector.color = color 
+            
+            
         
     @on_trait_change( 'selector:color' )
     def _color_changed ( self, color ):
@@ -199,7 +216,7 @@ class TextColorEditor ( BaseSimpleEditor ):
         if self.control.HasCapture():
             self.control.ReleaseMouse()
         
-        self.object.edit_traits( view = View( 
+        ui = self.object.edit_traits( view = View( 
                 Item(
                     self.name, 
                     id         = 'color_editor',
@@ -209,7 +226,14 @@ class TextColorEditor ( BaseSimpleEditor ):
                 ),
                 kind    = 'popup' ),
             parent = self.control )
-                
+
+        # PopupColorEditor's editor klass is a UIEditor, and we need to link
+        # its contained ui's history to the main history to get 'Revert' and 
+        # 'Cancel' actions to work right.
+        # FIXME: Is something like this needed in the UIEditor class itself?
+        if ui._editors[0].editor_ui.history is None:
+            ui._editors[0].editor_ui.history = self.ui.history
+
     #---------------------------------------------------------------------------
     #  Creates the control to use for the simple editor:
     #---------------------------------------------------------------------------
@@ -238,7 +262,7 @@ class TextColorEditor ( BaseSimpleEditor ):
         """ Updates the editor when the object trait changes externally to the 
             editor.
         """
-        self.color_sample.text = self.str_value
+        self.color_sample.text = str(self.str_value)
         set_color( self )
 
     #---------------------------------------------------------------------------
@@ -248,7 +272,11 @@ class TextColorEditor ( BaseSimpleEditor ):
     def string_value ( self, color ):
         """ Returns the text representation of a specified color value.
         """
-        return self.factory.str_color( color ) 
+        if self.factory.show_text:
+            str = self.factory.str_color( color )
+        else:
+            str = ''
+        return str 
 
 #-------------------------------------------------------------------------------
 #  'ReadonlyColorEditor' class:
