@@ -74,7 +74,7 @@ class SimpleEditor ( Editor ):
     kind = Str
 
     # Is the list of items being edited mutable?
-    mutable = Bool( True )
+    mutable = Bool
 
     # The image used by the editor:
     image = Image( 'list_editor' )
@@ -141,9 +141,13 @@ class SimpleEditor ( Editor ):
         self._editor = getattr( editor, self.kind )
 
         # Set up the additional 'list items changed' event handler needed for
-        # a list based trait:
+        # a list based trait. Note that we want to fire the update_editor_item
+        # only when the items in the list change and not when intermediate 
+        # traits change. Therefore, replace "." by ":" in the extended_name
+        # when setting up the listener.
+        extended_name = self.extended_name.replace('.', ':')
         self.context_object.on_trait_change( self.update_editor_item,
-                               self.extended_name + '_items?', dispatch = 'ui' )
+                               extended_name + '_items?', dispatch = 'ui' )
         self.set_tooltip()
 
     #---------------------------------------------------------------------------
@@ -153,8 +157,9 @@ class SimpleEditor ( Editor ):
     def dispose ( self ):
         """ Disposes of the contents of an editor.
         """
+        extended_name = self.extended_name.replace('.', ':')
         self.context_object.on_trait_change( self.update_editor_item,
-                                 self.extended_name + '_items?', remove = True )
+                                 extended_name + '_items?', remove = True )
         self._dispose_items()
 
         super( SimpleEditor, self ).dispose()
@@ -236,10 +241,10 @@ class SimpleEditor ( Editor ):
         list_pane.SetSizer( list_sizer )
 
         if is_fake:
-           self._cur_control = control
-           self.empty_list()
-           control.Destroy()
-           pcontrol.Destroy()
+            self._cur_control = control
+            self.empty_list()
+            control.Destroy()
+            pcontrol.Destroy()
 
         rows = 1
         if not self.single_row:
@@ -264,7 +269,7 @@ class SimpleEditor ( Editor ):
     #  the editor:
     #---------------------------------------------------------------------------
 
-    def update_editor_item ( self, event ):
+    def update_editor_item ( self, obj, name, event ):
         """ Updates the editor when an item in the object trait changes
         externally to the editor.
         """
@@ -479,7 +484,11 @@ class SimpleEditor ( Editor ):
         """ Returns a default value for the 'kind' trait.
         """
         return self.factory.style + '_editor'
-    
+        
+    def _mutable_default(self):
+        """ Trait handler to set the mutable trait from the factory. """
+        return self.factory.mutable
+
 #-------------------------------------------------------------------------------
 #  'CustomEditor' class:
 #-------------------------------------------------------------------------------
@@ -505,6 +514,7 @@ class CustomEditor ( SimpleEditor ):
     # Is the list editor is scrollable? This values overrides the default.
     scrollable = True
 
+
 #-------------------------------------------------------------------------------
 #  'TextEditor' class:
 #-------------------------------------------------------------------------------
@@ -589,14 +599,13 @@ class NotebookEditor ( Editor ):
         dock_controls = []
         for object in self.value:
             dock_control, view_object, monitoring = self._create_page( object )
-
             # Remember the DockControl for later deletion processing:
             uis.append( [ dock_control, object, view_object, monitoring ] )
-
             dock_controls.append( dock_control )
 
         # Add the new items to the DockWindow:
         self.add_controls( dock_controls )
+
         if self.ui.info.initialized:
             self.update_layout()
 
@@ -625,7 +634,6 @@ class NotebookEditor ( Editor ):
 
         # Add a page for each added object:
         dock_controls = []
-        first_control = None
         for object in event.added:
             dock_control, view_object, monitoring  = self._create_page( object )
             self._uis[ index: index ] = [ [ dock_control, object, view_object,
@@ -683,8 +691,10 @@ class NotebookEditor ( Editor ):
                 section.contents.append( DockRegion( contents = controls ) )
             else:
                 for control in controls:
-                    section.contents[-1].add(control, activate = 
-                                             self.selected is None)
+                    section.contents[-1].add(control, activate=False)
+            # Fire this event to activate the dock control corresponding
+            # to the selected object, if any.
+            self._selected_changed(None, self.selected)
 
     #---------------------------------------------------------------------------
     #  Updates the layout of the DockWindow:
@@ -790,9 +800,22 @@ class NotebookEditor ( Editor ):
                                     dockable  = DockableListElement(
                                                     ui     = ui,
                                                     editor = self ) )
-
         return ( dock_control, view_object, monitoring )
 
+    #---------------------------------------------------------------------------
+    #  Activates the corresponding dock window when the 'selected' trait of
+    #  the editor is changed.
+    #---------------------------------------------------------------------------
+    def _selected_changed(self, old, new):
+        """ Activates the corresponding dock window when the 'selected' trait 
+        of the editor is changed.
+        """
+        for i, value in enumerate(self._uis):
+            if new == value[1]:
+                value[0].activate()
+                break
+        return
+                        
 #-------------------------------------------------------------------------------
 #  'DockableListElement' class:
 #-------------------------------------------------------------------------------
