@@ -73,6 +73,9 @@ class SimpleSliderEditor ( Editor ):
     # Formatting string used to format value and labels
     format = Str
 
+    # Function to evaluate textual user input
+    evaluate = Any
+
     # Flag indicating that the UI is in the process of being updated
     ui_changing = Bool( False )
 
@@ -93,30 +96,30 @@ class SimpleSliderEditor ( Editor ):
             self.high = factory.high
 
         self.format = factory.format
-        size        = wx.DefaultSize
 
+        self.evaluate = factory.evaluate
+        self.sync_value( factory.evaluate_name, 'evaluate', 'from' )
+
+        size        = wx.DefaultSize
         if factory.label_width > 0:
             size = wx.Size( factory.label_width, 20 )
 
         self.sync_value( factory.low_name,  'low',  'from' )
         self.sync_value( factory.high_name, 'high', 'from' )
-        low  = self.low
-        high = self.high
+        #low  = self.low
+        #high = self.high
         self.control = panel = TraitsUIPanel( parent, -1 )
         sizer  = wx.BoxSizer( wx.HORIZONTAL )
         fvalue = self.value
 
         try:
             fvalue_text = self.format % fvalue
-            1 / (low <= fvalue <= high)
+            1 / (self.low <= fvalue <= self.high)
         except:
             fvalue_text = ''
             fvalue      = low
 
-        if high > low:
-            ivalue = int( (float( fvalue - low ) / (high - low)) * 10000 )
-        else:
-            ivalue = low
+        ivalue = self._convert_to_slider(fvalue)
 
         self._label_lo = wx.StaticText( panel, -1, '999999', size = size,
                                 style = wx.ALIGN_RIGHT | wx.ST_NO_AUTORESIZE )
@@ -132,21 +135,12 @@ class SimpleSliderEditor ( Editor ):
         self._label_hi = wx.StaticText( panel, -1, '999999', size = size )
         sizer.Add( self._label_hi, 0, wx.ALIGN_CENTER )
 
-        if factory.enter_set:
-            panel.text = text = wx.TextCtrl( panel, -1, fvalue_text,
-                                             size  = wx.Size( 56, 20 ),
-                                             style = wx.TE_PROCESS_ENTER )
-            wx.EVT_TEXT_ENTER( panel, text.GetId(),
-                               self.update_object_on_enter )
-        else:
-            panel.text = text = wx.TextCtrl( panel, -1, fvalue_text,
-                                             size  = wx.Size( 56, 20 ) )
-
+        panel.text = text = wx.TextCtrl( panel, -1, fvalue_text,
+                                         size  = wx.Size( 56, 20 ),
+                                         style = wx.TE_PROCESS_ENTER )
+        wx.EVT_TEXT_ENTER( panel, text.GetId(), self.update_object_on_enter )
         wx.EVT_KILL_FOCUS( text, self.update_object_on_enter )
         
-        if factory.auto_set:
-           wx.EVT_TEXT( panel, text.GetId(), self.update_object_on_enter )
-
         sizer.Add( text, 0, wx.LEFT | wx.EXPAND, 4 )
 
         low_label = factory.low_label
@@ -174,14 +168,13 @@ class SimpleSliderEditor ( Editor ):
     def update_object_on_scroll ( self, event ):
         """ Handles the user changing the current slider value.
         """
-        value = self.low + ((float( event.GetPosition() ) / 10000.0) *
-                            (self.high - self.low))
-        if not self.factory.is_float:
-            value = int( value )
+        value = self._convert_from_slider(event.GetPosition())
         event_type = event.GetEventType()
         if ((event_type == wx.wxEVT_SCROLL_ENDSCROLL) or
             (self.factory.auto_set and
-             (event_type == wx.wxEVT_SCROLL_THUMBTRACK))):
+             (event_type == wx.wxEVT_SCROLL_THUMBTRACK)) or
+            (self.factory.enter_set and 
+             (event_type == wx.wxEVT_SCROLL_THUMBRELEASE))):
             try:
                 self.ui_changing = True
                 self.control.text.SetValue( self.format % value )
@@ -200,20 +193,17 @@ class SimpleSliderEditor ( Editor ):
         """
         try:
             try:
-                value = eval( self.control.text.GetValue().strip() )
+                value = self.evaluate( self.control.text.GetValue().strip() )
             except Exception, ex:
                 # The entered something that didn't eval as a number, (e.g., 'foo')
-                # pretend it didn't happen
+                # pretend it didn't happen (i.e. do not change self.value)
                 value = self.value
                 self.control.text.SetValue( str( value ) )
 
-            if not self.factory.is_float:
-                value = int( value )
             self.value = value
             if not self.ui_changing:
                 self.control.slider.SetValue(
-                    int( ((float( value ) - self.low) /
-                         (self.high - self.low)) * 10000 ) )
+                     self._convert_to_slider(self.value) )
             self.control.text.SetBackgroundColour(OKColor)
             self.control.text.Refresh()
             if self._error is not None:
@@ -244,22 +234,36 @@ class SimpleSliderEditor ( Editor ):
             editor.
         """
         value = self.value
-        low   = self.low
-        high  = self.high
         try:
             text = self.format % value
-            1 / (low <= value <= high)
+            1 / (self.low <= value <= self.high)
         except:
             text  = ''
-            value = low
-
-        if high > low:
-            ivalue = int( (float( value - low ) / (high - low)) * 10000.0 )
-        else:
-            ivalue = low
-
+            value = self.low
+        
+        ivalue = self._convert_to_slider( value )
         self.control.text.SetValue( text )
         self.control.slider.SetValue( ivalue )
+
+    def _convert_to_slider(self, value):
+        """ Returns the slider setting corresponding to the user-supplied value.
+        """
+        if self.high > self.low:
+            ivalue = int( (float( value - self.low ) / 
+                           (self.high - self.low)) * 10000.0 )
+        else:
+            ivalue = self.low
+        return ivalue
+
+    def _convert_from_slider(self, ivalue):
+        """ Returns the float or integer value corresponding to the slider 
+        setting.
+        """
+        value = self.low + ((float( ivalue ) / 10000.0) *
+                            (self.high - self.low))
+        if not self.factory.is_float:
+            value = int(round(value))
+        return value
             
     #---------------------------------------------------------------------------
     #  Returns the editor's control for indicating error status:
@@ -296,6 +300,33 @@ class SimpleSliderEditor ( Editor ):
             self._label_hi.SetLabel( self.format % high  )
             self.update_editor()
 
+
+#-------------------------------------------------------------------------------
+class LogRangeSliderEditor ( SimpleSliderEditor ):
+#-------------------------------------------------------------------------------
+    """ A slider editor for log-spaced values 
+    """
+
+    def _convert_to_slider(self, value):
+        """ Returns the slider setting corresponding to the user-supplied value.
+        """
+        value = max(value, self.low)
+        ivalue = int( (log10(value) - log10(self.low)) /
+                      (log10(self.high) - log10(self.low)) * 10000.0)
+        return ivalue
+
+    def _convert_from_slider(self, ivalue):
+        """ Returns the float or integer value corresponding to the slider 
+        setting.
+        """
+        value = float( ivalue ) / 10000.0 * (log10(self.high) -log10(self.low))
+        # Do this to handle floating point errors, where fvalue may exceed 
+        # self.high.
+        fvalue = min(self.low*10**(value), self.high)
+        if not self.factory.is_float:
+            fvalue = int(round(fvalue))
+        return fvalue
+
 #-------------------------------------------------------------------------------
 #  'LargeRangeSliderEditor' class:
 #-------------------------------------------------------------------------------
@@ -324,6 +355,9 @@ class LargeRangeSliderEditor ( Editor ):
     # High end of displayed range
     cur_high = Float
 
+    # Function to evaluate textual user input
+    evaluate = Any
+
     # Flag indicating that the UI is in the process of being updated
     ui_changing = Bool( False )
 
@@ -341,10 +375,12 @@ class LargeRangeSliderEditor ( Editor ):
         # Initialize using the factory range defaults:
         self.low = factory.low
         self.high = factory.high
+        self.evaluate = factory.evaluate
 
         # Hook up the traits to listen to the object.
         self.sync_value( factory.low_name,  'low',  'from' )
         self.sync_value( factory.high_name, 'high', 'from' )
+        self.sync_value( factory.evaluate_name, 'evaluate', 'from' )
 
         self.init_range()
         low  = self.cur_low
@@ -362,7 +398,7 @@ class LargeRangeSliderEditor ( Editor ):
             fvalue      = low
 
         if high > low:
-           ivalue = int( (float( fvalue - low ) / (high - low)) * 10000 )
+            ivalue = int( (float( fvalue - low ) / (high - low)) * 10000 )
         else:
             ivalue = low
 
@@ -405,20 +441,11 @@ class LargeRangeSliderEditor ( Editor ):
         sizer.Add( label_hi, 2, wx.ALIGN_CENTER )
 
         # Text entry:
-        if factory.enter_set:
-            panel.text = text = wx.TextCtrl( panel, -1, fvalue_text,
-                                             size  = wx.Size( 56, 20 ),
-                                             style = wx.TE_PROCESS_ENTER )
-            wx.EVT_TEXT_ENTER( panel, text.GetId(),
-                               self.update_object_on_enter )
-        else:
-            panel.text = text = wx.TextCtrl( panel, -1, fvalue_text,
-                                             size  = wx.Size( 56, 20 ) )
-
+        panel.text = text = wx.TextCtrl( panel, -1, fvalue_text,
+                                         size  = wx.Size( 56, 20 ),
+                                         style = wx.TE_PROCESS_ENTER )
+        wx.EVT_TEXT_ENTER( panel, text.GetId(), self.update_object_on_enter )
         wx.EVT_KILL_FOCUS( text, self.update_object_on_enter )
-        
-        if factory.auto_set:
-           wx.EVT_TEXT( panel, text.GetId(), self.update_object_on_enter )
 
         sizer.Add( text, 0, wx.LEFT | wx.EXPAND, 4 )
 
@@ -451,7 +478,9 @@ class LargeRangeSliderEditor ( Editor ):
             self.ui_changing = True
             if ((event_type == wx.wxEVT_SCROLL_ENDSCROLL) or
                 (self.factory.auto_set and
-                 (event_type == wx.wxEVT_SCROLL_THUMBTRACK))):
+                 (event_type == wx.wxEVT_SCROLL_THUMBTRACK)) or
+                (self.factory.enter_set and 
+                 (event_type == wx.wxEVT_SCROLL_THUMBRELEASE))):
                 if self.factory.is_float:
                     self.value = value
                 else:
@@ -467,7 +496,7 @@ class LargeRangeSliderEditor ( Editor ):
         """ Handles the user pressing the Enter key in the text field.
         """
         try:
-            self.value = value = eval( self.control.text.GetValue().strip() )
+            self.value = value = self.evaluate( self.control.text.GetValue().strip() )
             self.control.text.SetBackgroundColour(OKColor)
             self.control.text.Refresh()
             # Update the slider range.
@@ -783,7 +812,7 @@ class RangeTextEditor ( TextEditor ):
         wx.EVT_KILL_FOCUS( control, self.update_object )
         
         if self.factory.auto_set:
-           wx.EVT_TEXT( parent, control.GetId(), self.update_object )
+            wx.EVT_TEXT( parent, control.GetId(), self.update_object )
            
         self.control = control
         self.set_tooltip()
@@ -857,7 +886,8 @@ SimpleEditorMap = {
     'xslider': LargeRangeSliderEditor,
     'spinner': SimpleSpinEditor,
     'enum':    SimpleEnumEditor,
-    'text':    RangeTextEditor
+    'text':    RangeTextEditor,
+    'logslider':     LogRangeSliderEditor
 }
 # Mapping between editor factory modes and custom editor classes
 CustomEditorMap = {
@@ -865,7 +895,8 @@ CustomEditorMap = {
     'xslider': LargeRangeSliderEditor,
     'spinner': SimpleSpinEditor,
     'enum':    CustomEnumEditor,
-    'text':    RangeTextEditor
+    'text':    RangeTextEditor,
+    'logslider':     LogRangeSliderEditor
 }
 
 ### EOF #######################################################################
