@@ -50,7 +50,7 @@ from enthought.traits.ui.table_filter \
 from enthought.traits.ui.ui_traits \
     import SequenceTypes
 
-from enthought.pyface.grid.api \
+from enthought.pyface.ui.wx.grid.api \
     import Grid
 
 from enthought.pyface.dock.api \
@@ -358,6 +358,18 @@ class TableEditor ( Editor ):
         _grid.SetMinSize( wx.Size(
             max( 150, 80 * len( self.columns ) ), 
             _grid.GetColLabelSize() + ((factory.rows or 15) * height) ) )
+
+        # On Linux, there is what appears to be a bug in wx in which the
+        # vertical scrollbar will not be sized properly if the TableEditor is 
+        # sized to be shorter than the minimum height specified above. Since
+        # this height is only set to ensure that the TableEditor is sized 
+        # correctly during the initial UI layout, we un-set it after this takes
+        # place (addresses ticket 1810)
+        def clear_minimum_height ( info ):
+            min_size = _grid.GetMinSize()
+            min_size.height = 0
+            _grid.SetMinSize ( min_size )
+        self.ui.add_defined ( clear_minimum_height )
         
         sizer.Add( grid.control, 1, wx.EXPAND ) 
         
@@ -459,7 +471,7 @@ class TableEditor ( Editor ):
         """
         if not isinstance( objects, SequenceTypes ):
             objects = [ objects ]
-            
+
         self.grid.set_selection( [ TraitGridSelection( obj = object )
                                    for object in objects ], notify = notify )
 
@@ -473,10 +485,11 @@ class TableEditor ( Editor ):
         """
         if (len( pairs ) == 1) and isinstance( pairs[0], list ):
             pairs = pairs[0]
-            
-        self.grid.set_selection(
-            [ TraitGridSelection( obj = object, name = name )
-              for object, name in pairs ] )
+
+        grid_selections = [TraitGridSelection(obj = object, name = name)
+                           for object, name in pairs] 
+           
+        self.grid.set_selection(grid_selections)
 
     #---------------------------------------------------------------------------
     #  Creates a new row object using the provided factory:
@@ -911,9 +924,8 @@ class TableEditor ( Editor ):
         values.sort( lambda l, r: cmp( l[0], r[0] ) )                
          
         # Save the new selection information:
-        self.set( selected_cell_indices = [ v[0] for v in values ],
-                  trait_change_notify   = False )
-        cells = [ v[1] for v in values ]        
+        self.setx( selected_cell_indices = [ v[0] for v in values ])
+        cells = [ v[1] for v in values ]  
         self.setx( selected_cells = cells )
             
         # Update the toolbar status:
@@ -988,8 +1000,8 @@ class TableEditor ( Editor ):
         if not self._no_notify:
             value = self.value
             gcn   = self.model.get_column_name
-            self.set_extended_selection( [ ( value[ row ], gcn( column ) ) 
-                                           for row, column in pairs ] )
+            new_selection = [(value[row], gcn(col)) for row, col in pairs] 
+            self.set_extended_selection(new_selection)
         
     def _update_toolbar ( self, has_selection ):
         """ Updates the toolbar after a selection change.
@@ -1890,5 +1902,8 @@ class TableSearchHandler ( Handler ):
 # Define the SimpleEditor class.
 SimpleEditor = TableEditor
 
+# Define the ReadonlyEditor class.
+ReadonlyEditor = TableEditor
+
 ### EOF #######################################################################
 
