@@ -42,10 +42,10 @@ from enthought.traits.ui.table_filter \
 from enthought.traits.ui.ui_traits \
     import ATheme, SequenceTypes
 
-from enthought.pyface.grid.api \
+from enthought.pyface.ui.wx.grid.api \
     import GridModel, GridSortEvent
 
-from enthought.pyface.grid.trait_grid_cell_adapter \
+from enthought.pyface.ui.wx.grid.trait_grid_cell_adapter \
     import TraitGridCellAdapter
 
 from enthought.pyface.timer.api \
